#!/usr/bin/perl
use strict;
use warnings;
use Test::More qw(no_plan);
use RDF::Redland;

use_ok( 'RDF::Query' );

my $data	= RDF::Redland::URI->new( 'file://' . File::Spec->rel2abs( 'data/foaf.xrdf' ) );
my $storage	= new RDF::Redland::Storage("hashes", "test", "new='yes',hash-type='memory'");
my $model	= new RDF::Redland::Model($storage, "");
my $parser	= new RDF::Redland::Parser("rdfxml");
$parser->parse_into_model($data, $data, $model);

my $bridge	= RDF::Query::Model::Redland->new( $model );
is( $bridge->count, 53, '53 Statements in Model' );

{
	my $query		= new RDF::Query ( <<"END", undef, undef, 'rdql' );
		SELECT
			?person
		WHERE
			(?person foaf:name "Gregory Todd Williams")
		USING
			foaf FOR <http://xmlns.com/foaf/0.1/>
END
	my ($person)	= $query->get( $model );
 	ok( $query->bridge->isa_resource( $person ), 'Resource' );
 	is( $person->getLabel, 'http://kasei.us/about/foaf.xrdf#greg', 'Person uri' );
}

{
	print "# Triple with QName subject\n";
	my $query		= new RDF::Query ( <<"END", undef, undef, 'rdql' );
		SELECT
			?name
		WHERE
			(kasei:greg foaf:name ?name)
		USING
			kasei FOR <http://kasei.us/about/foaf.xrdf#>
			foaf FOR <http://xmlns.com/foaf/0.1/>
END
	my ($name)	= $query->get( $model );
 	ok( $query->bridge->isa_literal( $name ), 'Literal' );
 	is( $name->getLabel, 'Gregory Todd Williams', 'Person name' );
}

{
	print "# Early triple with multiple unbound variables\n";
	my $query		= new RDF::Query ( <<"END", undef, undef, 'rdql' );
		SELECT
			?person ?name
		WHERE
			(?person foaf:name ?name)
			(?person foaf:homepage <http://kasei.us/>)
		USING
			foaf FOR <http://xmlns.com/foaf/0.1/>
END
	my @results	= $query->execute( $model );
 	ok( $query->bridge->isa_resource( $results[0][0] ), 'Person Resource' );
 	ok( $query->bridge->isa_literal( $results[0][1] ), 'Name Resource' );
 	is( $results[0][0]->getLabel, 'http://kasei.us/about/foaf.xrdf#greg', 'Person uri' );
 	is( $results[0][1]->getLabel, 'Gregory Todd Williams', 'Person name' );
}

{
	print "# Triple with no variable, present in data\n";
	my $query		= new RDF::Query ( <<"END", undef, undef, 'rdql' );
		SELECT
			?person
		WHERE
			(?person foaf:name "Gregory Todd Williams")
			(<http://kasei.us/about/foaf.xrdf#greg> foaf:homepage <http://kasei.us/>)
		USING
			foaf FOR <http://xmlns.com/foaf/0.1/>
END
	my @results	= $query->execute( $model );
 	ok( $query->bridge->isa_resource( $results[0][0] ), 'Person Resource' );
 	is( $results[0][0]->getLabel, 'http://kasei.us/about/foaf.xrdf#greg', 'Person uri' );
}

{
	print "# Triple with no variable, not present in data\n";
	my $query		= new RDF::Query ( <<"END", undef, undef, 'rdql' );
		SELECT
			?person
		WHERE
			(?person foaf:name "Gregory Todd Williams")
			(<http://localhost/greg> foaf:homepage <http://kasei.us/>)
		USING
			foaf FOR <http://xmlns.com/foaf/0.1/>
END
	my @results	= $query->execute( $model );
	is( scalar(@results), 0, 'No data returned for bogus triple' );
}

{
	print "# Broken query with too few triples\n";
	my $query		= new RDF::Query ( <<"END", undef, undef, 'rdql' );
		SELECT
			?person
		WHERE
			(?person foaf:name ?name)
		USING
			foaf FOR <http://xmlns.com/foaf/0.1/>
END
	my @results	= $query->execute( $model );
	is( $results[0], undef, 'Error (undef row) on too few triples' );
}

{
	print "# Broken query triple (variable with missing '?')\n";
	my $query		= new RDF::Query ( <<"END", undef, undef, 'rdql' );
		SELECT
			?person
		WHERE
			(person foaf:name "Gregory Todd Williams")
		USING
			foaf FOR <http://xmlns.com/foaf/0.1/>
END
	my @results	= $query->execute( $model );
	is( $results[0], undef, 'Error (undef row) on no triples' );
}

{
	print "# Backend tests\n";
	my $query		= new RDF::Query ( <<"END", undef, undef, 'rdql' );
		SELECT
			?name ?homepage
		WHERE
			(kasei:greg foaf:name ?name)
			(kasei:greg foaf:homepage ?homepage)
		USING
			kasei FOR <http://kasei.us/about/foaf.xrdf#>
			foaf FOR <http://xmlns.com/foaf/0.1/>
END
	my ($name,$homepage)	= $query->get( $model );

	ok( !$query->bridge->isa_resource( 0 ), 'isa_resource(0)' );
	ok( !$query->bridge->isa_resource( $name ), 'isa_resource(literal)' );
	ok( $query->bridge->isa_resource( $homepage ), 'isa_resource(resource)' );

	ok( !$query->bridge->isa_literal( 0 ), 'isa_literal(0)' );
	ok( !$query->bridge->isa_literal( $homepage ), 'isa_literal(resource)' );
	ok( $query->bridge->isa_literal( $name ), 'isa_literal(literal)' );
}

{
 	my $query	= new RDF::Query ();
	print "# AUTOLOAD tests\n";
	eval { $query->unimplemented; };
	like( $@, qr/Can't locate object method "unimplemented" via package/, 'AUTOLOAD avoiding bogus instance data' );
	is( RDF::Query->unimplemented, undef, 'AUTOLOAD avoiding class method calls' );
}
